@@ -1,17 +1,17 @@
-{-# LANGUAGE OverloadedStrings #-}
-
 --
 -- Test with threads
 --
 
--- module TestWithThreads where
-
 import qualified TestOrdECMWithThreads as OrdTest
 import qualified TestHashECMWithThreads as HashTest
+import qualified TestOrdECMWithThreadsInvalidating as OrdTestInvalidating
+import qualified TestHashECMWithThreadsInvalidating as HashTestInvalidating
 
 testWithThreads = do
   HashTest.testWithThreads
   OrdTest.testWithThreads
+  HashTestInvalidating.testWithThreadsInvalidating
+  OrdTestInvalidating.testWithThreadsInvalidating
   return ()
 
 main = testWithThreads
